unit UnitMDC;
{
Unit: UnitMDC
Programmed By  : Stucuk
Description: This unit is based on the pdf document 'The MDC File Format' by Wolfgang Prinz

Copyright  2004 Stuart Carey
}

interface

Uses Math3d;

type
TMDCFileHeader = record
ulIndent : array [0..3] of char; {IDPC} // Note: its a Longword in the definition!
ulVersion : Longword; {2}
ucName : array [0..63] of Char;
ulFlags,ulNumFrames,ulNumTags,ulNumSurfaces,
ulNumSkins,ulOffsetBorderFrames,ulOffsetTagNames,
ulOffsetTagFrames,ulOffsetSurfaces,ulOffsetEnd : Longword;
end;

TMDCFrame = record
bboxMin,bboxMax,localOrigin : TVector3f;
radius : single;
name : array [0..15] of char;
end;

TMDCTagName = record
ucName : array [0..63] of char;
end;

TMDCTagFrame = record
tpPosition, taAngle : TVector3b2;
end;

TMDCSurfaceHeader = record
ulIndent : Longword;
ucName : array [0..63] of Char;
ulFlags,ulNumCompFrames,ulNumBaseFrames,ulNumShaders,
ulNumVertices,ulNumTriangles,ulOffsetTriangles,ulOffsetShaders,
ulOffsetTexCoords,ulOffsetBaseVerts,ulOffsetCompVerts,
ulOffsetFrameBaseFrames,ulOffsetFrameCompFrames,ulOffsetEnd : Longword;
end;

TMDCTriangle = record
ulVertex1,ulVertex2,ulVertex3 : longword;
end;

TMDCShader = record
ucName : array [0..63] of Char;
ulFlags : longword;
end;

TTexCoord = record
s,t : single;
end;

TBaseVertex = record
x,y,z : shortint;
Normal : Byte;
end;

TVertex = record
x,y,z : single;
Normal : Byte;
end;

TCompVertex = record {WARNING MAY BE WRONG!!!}
x,y,z,n : Byte; //unsigned char
end;


//Own Declerations

TTag = Record
Name : TMDCTagName;
Frame : TMDCTagFrame;
end;

TMDCSurface = Record // This will hold the information needed for 1 Surface
Header : TMDCSurfaceHeader;

Shaders : Array of TMDCShader;

FrameBaseFrames,FrameCompFrames : Array of Byte; // Not needed after vertex is processed!
BaseVertex : Array of TBaseVertex; // Not needed after vertex is processed!
CompVertex : Array of TCompVertex; // Not needed after vertex is processed!
Vertex : Array of TVertex; // Processed Vertex!

Triangle : Array of TMDCTriangle;
TexCoord : Array of TTexCoord;
end;

TMDCFile = Record // This will hold the information needed for 1 MDC file
Header : TMDCFileHeader;

Frames : Array of TMDCFrame;
Surfaces : Array of TMDCSurface;
TagName : Array of TMDCTagName;
TagFrame : Array of TMDCTagFrame;
// NOTE: CAN'T FIND SKIN DEFINITION IN THE DOCUMENT!!!!!

Loaded: Boolean; //Incase its not loaded the program can check this!
Frame : Cardinal;
end;

Procedure LoadMDC(Var Model : TMDCFile; Const Filename : String);
Procedure DrawMDC(Model : TMDCFile; Position : TVector3f; Angles : TVector2f);

implementation

Uses Windows,Sysutils,OpenGL15;

Procedure ProcessVertex(Var Surface : TMDCSurface; Const Model : TMDCFile);
var
x : integer;
VBase,VDelta,VLocalOrigin : TVector3f;
begin

for x := 0 to Surface.Header.ulNumVertices-1 do
begin
try
If Surface.Header.ulNumBaseFrames > 0 then
VLocalOrigin := Model.Frames[Surface.FrameBaseFrames[Model.frame]].localOrigin;

VDelta := SetVector(0,0,0);

If Surface.Header.ulNumCompFrames > 0 then
If Surface.FrameCompFrames[Model.frame] <> $FFFF then
begin
VDelta.X := Surface.CompVertex[x].X * (1/64) * 4;
VDelta.Y := Surface.CompVertex[x].Y * (1/64) * 4;
VDelta.Z := Surface.CompVertex[x].Z * (1/64) * 4;
VLocalOrigin := Model.Frames[Surface.FrameCompFrames[Model.frame]].localOrigin;
end;

VBase.X := Surface.BaseVertex[x].X * (1/64);
VBase.Y := Surface.BaseVertex[x].Y * (1/64);
VBase.Z := Surface.BaseVertex[x].Z * (1/64);

Surface.Vertex[x].X := VLocalOrigin.X + VBase.X + VDelta.X;
Surface.Vertex[x].Y := VLocalOrigin.Y + VBase.Y + VDelta.Y;
Surface.Vertex[x].Z := VLocalOrigin.Z + VBase.Z + VDelta.Z;
except
end;
end;


end;

Procedure LoadMDCSurface(Var Surface : TMDCSurface; Var F : File; Const Pos : Integer);
Var
X : integer;
begin

SetLength(Surface.Shaders,Surface.Header.ulNumShaders);
SetLength(Surface.FrameBaseFrames,Surface.Header.ulNumBaseFrames);
SetLength(Surface.FrameCompFrames,Surface.Header.ulNumCompFrames);
SetLength(Surface.BaseVertex,Surface.Header.ulNumVertices);
SetLength(Surface.CompVertex,Surface.Header.ulNumVertices);
SetLength(Surface.Vertex,Surface.Header.ulNumVertices);
SetLength(Surface.Triangle,Surface.Header.ulNumTriangles);
SetLength(Surface.TexCoord,Surface.Header.ulNumVertices); // Num of Verticies?????

Seek(F,Pos + Surface.Header.ulOffsetShaders);

for x := 0 to Surface.Header.ulNumShaders-1 do
BlockRead(F,Surface.Shaders[x],SizeOf(TMDCShader));

Seek(F,Pos + Surface.Header.ulOffsetFrameBaseFrames);

If Surface.Header.ulNumBaseFrames > 0 then
for x := 0 to Surface.Header.ulNumBaseFrames-1 do
BlockRead(F,Surface.FrameBaseFrames[x],SizeOf(Byte));

Seek(F,Pos + Surface.Header.ulOffsetFrameCompFrames);

If Surface.Header.ulNumCompFrames > 0 then
for x := 0 to Surface.Header.ulNumCompFrames-1 do
BlockRead(F,Surface.FrameCompFrames[x],SizeOf(Byte));

Seek(F,Pos + Surface.Header.ulOffsetBaseVerts);

for x := 0 to Surface.Header.ulNumVertices-1 do
BlockRead(F,Surface.BaseVertex[x],SizeOf(TBaseVertex));

Seek(F,Pos + Surface.Header.ulOffsetCompVerts);

If Surface.Header.ulNumCompFrames > 0 then
for x := 0 to Surface.Header.ulNumVertices-1 do
BlockRead(F,Surface.CompVertex[x],SizeOf(TCompVertex));

Seek(F,Pos + Surface.Header.ulOffsetTriangles);

for x := 0 to Surface.Header.ulNumTriangles-1 do
BlockRead(F,Surface.Triangle[x],SizeOf(TMDCTriangle));

Seek(F,Pos + Surface.Header.ulOffsetTexCoords);

for x := 0 to Surface.Header.ulNumVertices-1 do
BlockRead(F,Surface.TexCoord[x],SizeOf(TTexCoord));

end;

Procedure LoadMDC(Var Model : TMDCFile; Const Filename : String);
Var
F : File;
X,P : integer;
begin


AssignFile(f, filename);
//Reset(f);
Reset(F,1);

Model.Loaded := False;

Try
BlockRead(f,Model.Header,sizeof(TMDCFileHeader));

//Clear Arrays
SetLength(Model.Frames,0);
SetLength(Model.Surfaces,0);
SetLength(Model.TagName,0);

If (Model.Header.ulIndent[0] <> 'I') or (Model.Header.ulIndent[1] <> 'D') or (Model.Header.ulIndent[2] <> 'P') or (Model.Header.ulIndent[3] <> 'C') or (Model.Header.ulVersion <> 2) then
begin
MessageBox(0,Pchar('Header Error:' + #13#13 + 'Filename: ' + Filename +#13 + 'ID: ' + Model.Header.ulIndent[0] + Model.Header.ulIndent[1] + Model.Header.ulIndent[2] + Model.Header.ulIndent[3] + #13 + 'Version: ' + inttostr(Model.Header.ulVersion)),'MDC Header Error',0);
Exit;
end;

//Set Array Lengths
SetLength(Model.Frames,Model.Header.ulNumFrames);
SetLength(Model.Surfaces,Model.Header.ulNumSurfaces);
SetLength(Model.TagName,Model.Header.ulNumTags);
SetLength(Model.TagFrame,Model.Header.ulNumTags);

Seek(F,Model.Header.ulOffsetBorderFrames);
for x := 0 to Model.Header.ulNumFrames-1 do
BlockRead(f,Model.Frames[x],sizeof(TMDCFrame));

Seek(F,Model.Header.ulOffsetSurfaces);
P := Model.Header.ulOffsetSurfaces{ + SizeOf(TMDCSurfaceHeader)};
for x := 0 to Model.Header.ulNumSurfaces-1 do
begin
BlockRead(f,Model.Surfaces[x],sizeof(TMDCSurfaceHeader));

If Model.Surfaces[x].Header.ulIndent <> 7 then
begin // Assume bad surface
Model.Header.ulNumSurfaces := x;
Break;
end;

LoadMDCSurface(Model.Surfaces[x],F,P);
P := P + Model.Surfaces[x].Header.ulOffsetEnd{ + SizeOf(TMDCSurfaceHeader)};

Seek(F,Model.Header.ulOffsetSurfaces + Model.Surfaces[x].Header.ulOffsetEnd);
end;

Seek(F,Model.Header.ulOffsetTagNames);
If model.Header.ulNumTags > 0 then
for x := 0 to Model.Header.ulNumTags-1 do
BlockRead(f,Model.TagName[x],sizeof(TMDCTagName));

Seek(F,Model.Header.ulOffsetTagFrames);
If model.Header.ulNumTags > 0 then
for x := 0 to Model.Header.ulNumTags-1 do
BlockRead(f,Model.TagFrame[x],sizeof(TMDCTagFrame));

{P := Model.Header.ulOffsetSurfaces + Model.Header.ulNumSurfaces*SizeOf(TMDCSurfaceHeader);
for x := 0 to Model.Header.ulNumSurfaces-1 do // Load all info required by the surface!
begin
LoadMDCSurface(Model.Surfaces[x],F,P);
P := P + Model.Surfaces[x].Header.ulOffsetEnd;
end;                            }

//Process Vertex's
for x := 0 to Model.Header.ulNumSurfaces-1 do
ProcessVertex(Model.Surfaces[x],Model);
                   {
MessageBox(0,Pchar('Header Error:' + #13#13 + 'Filename: ' + Filename +#13 + 'ID: ' + Model.Header.ulIndent[0] + Model.Header.ulIndent[1] + Model.Header.ulIndent[2] + Model.Header.ulIndent[3] + #13 + 'Version: ' + inttostr(Model.Header.ulVersion)),'MDC Header Error',0);
MessageBox(0,Pchar('Header:' + #13#13 + 'ID: ' + Model.Surfaces[0].Header.ulIndent[0] + Model.Surfaces[0].Header.ulIndent[1] + Model.Surfaces[0].Header.ulIndent[2] + Model.Surfaces[0].Header.ulIndent[3]),'MDC Header Error',0);
                   }
Model.Loaded := True;

Except
End;
Closefile(f);
end;

Procedure DrawTriangle(Vertex1,Vertex2,Vertex3 : TVertex; TexCoord1,TexCoord2,TexCoord3 : TTexCoord);
begin

glBegin(GL_TRIANGLES);

with TexCoord1 do
glTexCoord2f(s, t);
with Vertex1 do
glVertex3f(x, y, z);

with TexCoord2 do
glTexCoord2f(s, t);
with Vertex2 do
glVertex3f(x, y, z);

with TexCoord3 do
glTexCoord2f(s, t);
with Vertex3 do
glVertex3f(x, y, z);

glEnd;

end;

Procedure DrawMDCSurface(Surface : TMDCSurface);
var
x : integer;
begin

glBindTexture(GL_TEXTURE_2D,0);

for x := 0 to Surface.Header.ulNumTriangles-1 do
begin
DrawTriangle(Surface.Vertex[Surface.Triangle[x].ulVertex1],Surface.Vertex[Surface.Triangle[x].ulVertex2],Surface.Vertex[Surface.Triangle[x].ulVertex3],Surface.TexCoord[Surface.Triangle[x].ulVertex1],Surface.TexCoord[Surface.Triangle[x].ulVertex2],Surface.TexCoord[Surface.Triangle[x].ulVertex3]);
end;

end;

Procedure DrawMDC(Model : TMDCFile; Position : TVector3f; Angles : TVector2f);
var
x : integer;
begin

glLoadIdentity();
glTranslatef(Position.X,Position.Y,Position.Z);

glRotatef(Angles.X, 1, 0, 0); // pitch
glRotatef(Angles.Y, 0, 1, 0);  // roll

for x := 0 to Model.Header.ulNumSurfaces-1 do
DrawMDCSurface(Model.Surfaces[x]);

end;

end.
